@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -83,12 +84,18 @@ virtual		   ~CMessaging					( void );
 		
 		uInt32	GetServerVersion			( void );
 		void	SetServerVersion			( uInt32 inServerVersion );
-
+		const char	*GetProxyIPAddress		( void );
+#ifdef SERVERINTERNAL
+		static bool	IsThreadUsingInternalDispatchBuffering
+											( OSType inThreadSig );
+#endif
+		
 private:
 		sInt32	GetEmptyObj					( sComData *inMsg, eValueType inType, sObject **outObj );
 		sInt32	GetThisObj					( sComData *inMsg, eValueType inType, sObject **outObj );
+		sComData*   GetMsgData				( void );
 
-		void	Grow						( uInt32 inOffset, uInt32 inSize );
+		bool	Grow						( uInt32 inOffset, uInt32 inSize );
 
 #ifndef SERVERINTERNAL
 		CClientEndPoint	   *fCommPort;
