@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -35,7 +36,7 @@
 
 extern sInt32 gProcessPID;
 
-static const	uInt32	kBuffPad	= 16;
+static const	uInt32	kFWBuffPad	= 16;
 
 //--------------------------------------------------------------------------------------------------
 //
@@ -236,9 +237,9 @@ tDirStatus ExtractRecordEntry ( tDataBufferPtr		inOutDataBuff,
 		// Get the attribute count
 		::memcpy( &usAttrCnt, pData, 2 );
 
-		pRecEntry = (tRecordEntry *)::calloc( 1, sizeof( tRecordEntry ) + usNameLen + usTypeLen + 4 + kBuffPad );
+		pRecEntry = (tRecordEntry *)::calloc( 1, sizeof( tRecordEntry ) + usNameLen + usTypeLen + 4 + kFWBuffPad );
 
-		pRecEntry->fRecordNameAndType.fBufferSize	= usNameLen + usTypeLen + 4 + kBuffPad;
+		pRecEntry->fRecordNameAndType.fBufferSize	= usNameLen + usTypeLen + 4 + kFWBuffPad;
 		pRecEntry->fRecordNameAndType.fBufferLength	= usNameLen + usTypeLen + 4;
 
 		// Add the record name length
@@ -473,13 +474,13 @@ tDirStatus ExtractAttributeEntry (	tDataBufferPtr			inOutDataBuff,
 			}
 		}
 
-		uiAttrEntrySize = sizeof( tAttributeEntry ) + usAttrTypeLen + kBuffPad;
+		uiAttrEntrySize = sizeof( tAttributeEntry ) + usAttrTypeLen + kFWBuffPad;
 		pAttribInfo = (tAttributeEntry *)::calloc( 1, uiAttrEntrySize );
 
 		pAttribInfo->fAttributeValueCount				= usValueCnt;
 		pAttribInfo->fAttributeDataSize					= uiTotalValueSize;
 		pAttribInfo->fAttributeValueMaxSize				= 512;				// KW this is not used anywhere
-		pAttribInfo->fAttributeSignature.fBufferSize	= usAttrTypeLen + kBuffPad;
+		pAttribInfo->fAttributeSignature.fBufferSize	= usAttrTypeLen + kFWBuffPad;
 		pAttribInfo->fAttributeSignature.fBufferLength	= usAttrTypeLen;
 		::memcpy( pAttribInfo->fAttributeSignature.fBufferData, pAttrType, usAttrTypeLen );
 
@@ -668,9 +669,9 @@ tDirStatus ExtractAttributeValue (	tDataBufferPtr			 inOutDataBuff,
 
 		//if (usValueLen == 0)  throw( (sInt32)eDSInvalidBuffFormat ); //if zero is it okay?
 
-		pAttrValue = (tAttributeValueEntry *)::calloc( 1, sizeof( tAttributeValueEntry ) + usValueLen + kBuffPad );
+		pAttrValue = (tAttributeValueEntry *)::calloc( 1, sizeof( tAttributeValueEntry ) + usValueLen + kFWBuffPad );
 
-		pAttrValue->fAttributeValueData.fBufferSize		= usValueLen + kBuffPad;
+		pAttrValue->fAttributeValueData.fBufferSize		= usValueLen + kFWBuffPad;
 		pAttrValue->fAttributeValueData.fBufferLength	= usValueLen;
 		
 		// Do record check, verify that offset is not past end of buffer, etc.
