@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -51,8 +49,8 @@ static const char *sStaticPluginList[ kNumStaticPlugins ][ 2 ] =
 {
 	{ "Configure",	"1.6" },
 	{ "NetInfo",	"1.6" },
-	{ "LDAPv3",		"1.6.1" },
-	{ "Search",		"1.6" }
+	{ "LDAPv3",		"1.6.2" },
+	{ "Search",		"1.6.1" }
 };
 
 //--------------------------------------------------------------------------------------------------
